'''
Create a root CA, just-in-time provisioning role, and registration config on AWS IoT.
'''
import json
import os
from os import path

import boto3
from cryptography.hazmat.primitives import serialization

from .jitp_role import create_jitp_role
from ..crypto import \
    generate_ca_cert, \
    serialize_key_cert_pair, \
    generate_csr, \
    serialize_key_csr_pair, \
    generate_signed_cert
from ..file_system import basepath, configpath, get_home_directory


def init(thing_type, country='US', days=365, key_size=2048, log_level='DEBUG', **kwargs):
    """
    Creates a custom root CA and registers it with the cloud provider for automatic provisioning.
    """
    fqdn = kwargs.get('fqdn')
    organization = kwargs.get('organization')
    state = kwargs.get('state')
    locality = kwargs.get('locality')
    unit = kwargs.get('unit')

    root_ca_directory = path.join(configpath, thing_type)

    if path.isdir(root_ca_directory):
        raise Exception('A root CA for this thing type is already present at ' + root_ca_directory)

    IOT = boto3.client('iot')

    root_pair = generate_ca_cert(days, key_size)

    # aws iot get-registration-code
    # TODO: Make this client cloud-provider agnostic
    registration_code = IOT.get_registration_code().get("registrationCode", None)

    # Using this registration_code as common name
    csr_pair = generate_csr(key_size, registration_code, fqdn, organization, country, state, locality, unit)

    verification_cert = generate_signed_cert(csr_pair['csr'], root_pair, days)

    # This only happens on AWS
    jitp_role = create_jitp_role("wintergreen")

    registration_config = generate_registration_config(thing_type, jitp_role["role_arn"])

    serialized_ca_pair = serialize_key_cert_pair(root_pair)
    serialized_verification_pair = serialize_key_cert_pair({'cert': verification_cert['cert'], 'key': csr_pair['key']})

    ca_response = IOT.register_ca_certificate(
        caCertificate=serialized_ca_pair['cert'],
        verificationCertificate=serialized_verification_pair['cert'],
        setAsActive=True,
        allowAutoRegistration=True,
        registrationConfig=registration_config
    )

    logging_options_response = IOT.set_v2_logging_options(
        roleArn=jitp_role['role_arn'],
        defaultLogLevel=log_level
    )

    thing_type_response = IOT.create_thing_type(
        thingTypeName=thing_type
    )

    os.makedirs(root_ca_directory)

    root_ca_cert_path = path.join(root_ca_directory, 'rootCA.crt')
    root_ca_key_path = path.join(root_ca_directory, 'rootCA.key')

    with open(root_ca_cert_path, 'w') as f:
        f.write(serialized_ca_pair['cert'])

    with open(root_ca_key_path, 'w') as f:
        f.write(serialized_ca_pair['key'])

    output = {
        'thingTypeName': thing_type_response['thingTypeName'],
        'thingTypeArn': thing_type_response['thingTypeArn'],
        'thingTypeId': thing_type_response['thingTypeId'],
        'jitpRoleArn': jitp_role['role_arn'],
        'rootCACertPath': root_ca_cert_path,
        'rootCAKeyPath': root_ca_key_path,
        'certificateArn': ca_response['certificateArn']
    }

    IOT.create_thing_type(thingTypeName=thing_type)

    return output

def generate_registration_config(thing_type, role_arn):
    with open(path.join(basepath, 'templates', 'policy.json'), 'r') as policy:
        json_policy = json.loads(policy.read())

    with open(path.join(basepath, 'templates', 'provisioning.json'), 'r') as provisioning_file:
        pf = json.loads(provisioning_file.read())

    pf["Resources"]["thing"]["Properties"]["ThingTypeName"] = thing_type
    pf["Resources"]["policy"]["Properties"]["PolicyDocument"] = json.dumps(json_policy)

    return {
        "roleArn": role_arn,
        "templateBody": json.dumps(pf)
    }




"""
Handles crypto functions, creates certs
"""
import datetime
import ipaddress
from uuid import uuid4
from cryptography import x509
from cryptography.x509.oid import NameOID

from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes

ISSUER = x509.Name([
    x509.NameAttribute(NameOID.COMMON_NAME, u"wintergreen"),
])

def serialize_key_cert_pair(pair):
    return {'cert': pair['cert'].public_bytes(serialization.Encoding.PEM).decode('utf-8')
           ,'key': pair['key'].private_bytes(encoding=serialization.Encoding.PEM,
                        format=serialization.PrivateFormat.TraditionalOpenSSL,
                        encryption_algorithm=serialization.NoEncryption()).decode('utf-8')
           }

def serialize_key_csr_pair(pair):
    return {'csr': pair['csr'].public_bytes(serialization.Encoding.PEM).decode('utf-8')
           ,'key': pair['key'].private_bytes(encoding=serialization.Encoding.PEM,
                        format=serialization.PrivateFormat.TraditionalOpenSSL,
                        encryption_algorithm=serialization.NoEncryption()).decode('utf-8')
           }

def private_key(key_size):
    """
    Equivalent to openssl genrsa -out rootCA.key key_size
    """
    return rsa.generate_private_key(backend=default_backend(), public_exponent=65537, key_size=key_size)

def generate_ca_cert(root_ca_days, key_size):
    """
    Equivalent to
    openssl req -x509 -new -nodes -key rootCA.key -sha256 -days root_ca_days -out rootCA.pem
    """
    key = private_key(key_size)
    valid_until = datetime.datetime.utcnow() + datetime.timedelta(days=root_ca_days)
    cert = x509.CertificateBuilder().subject_name(
            ISSUER
        ).issuer_name(
            ISSUER
        ).public_key(
            key.public_key()
        ).serial_number(
            x509.random_serial_number()
        ).not_valid_before(
            datetime.datetime.utcnow()
        ).not_valid_after(
            valid_until
        ).add_extension(
            x509.BasicConstraints(ca=True, path_length=0)
            ,critical=True
        ).add_extension(
            x509.KeyUsage(
                digital_signature=True
                ,key_encipherment=False
                ,content_commitment=False
                ,data_encipherment=False
                ,key_agreement=False
                ,key_cert_sign=True
                ,crl_sign=True
                ,encipher_only=False
                ,decipher_only=False
                )
            ,critical=True
        ).sign(key, hashes.SHA256(), default_backend())

    return {'cert': cert, 'key': key}

def generate_csr(key_size, common_name, fqdn, organization, country, state, locality, unit):
    """
    Equivalent to
    openssl req -new -key verificationCert.key -out verificationCert.csr -config provisioning.cnf
    """
    key = private_key(key_size)
    csr = x509.CertificateSigningRequestBuilder().subject_name(
        x509.Name([
            x509.NameAttribute(NameOID.COMMON_NAME, u"{common_name}".format(common_name=common_name)),
            x509.NameAttribute(NameOID.ORGANIZATION_NAME, u"{organization}".format(organization=organization)),
            x509.NameAttribute(NameOID.COUNTRY_NAME, u"{country}".format(country=country)),
            x509.NameAttribute(NameOID.STATE_OR_PROVINCE_NAME, u"{state}".format(state=state)),
            x509.NameAttribute(NameOID.LOCALITY_NAME, u"{locality}".format(locality=locality)),
            x509.NameAttribute(NameOID.ORGANIZATIONAL_UNIT_NAME, u"{unit}".format(unit=unit))
        ])
    ).add_extension(
        x509.SubjectAlternativeName([
            x509.DNSName(u"{fqdn}".format(fqdn=fqdn)),
        ]),
        critical=False,
    ).sign(key, hashes.SHA256(), default_backend())
    return {'csr': csr, 'key': key}

def generate_signed_cert(csr, ca_pair, days):
    crt = x509.CertificateBuilder().subject_name(
        csr.subject
    ).issuer_name(
        ca_pair['cert'].subject
    ).public_key(
        csr.public_key()
    ).serial_number(
        uuid4().int  # pylint: disable=no-member
    ).not_valid_before(
        datetime.datetime.utcnow()
    ).not_valid_after(
        datetime.datetime.utcnow() + datetime.timedelta(days=days)
    ).add_extension(
        extension=x509.KeyUsage(
            digital_signature=True, key_encipherment=True, content_commitment=True,
            data_encipherment=False, key_agreement=False, encipher_only=False, decipher_only=False, key_cert_sign=False, crl_sign=False
        ),
        critical=True
    ).add_extension(
        extension=x509.BasicConstraints(ca=False, path_length=None),
        critical=True
    ).add_extension(
        extension=x509.AuthorityKeyIdentifier.from_issuer_public_key(ca_pair['key'].public_key()),
        critical=False
    ).sign(
        private_key=ca_pair['key'],
        algorithm=hashes.SHA256(),
        backend=default_backend()
    )
    return {'cert': crt}

def generate_cert(ca_key, device_name, days):
    """
    Using this common name
    openssl genrsa -out verificationCert.key 2048 -config provisioning.cnf
    openssl req -new -key verificationCert.key -out verificationCert.csr -config provisioning.cnf
    """
    key = private_key()
    valid_until = datetime.datetime.utcnow() + datetime.timedelta(days=root_ca_days)
    cert = x509.CertificateBuilder().subject_name(
            x509.Name([
                x509.NameAttribute(NameOID.COMMON_NAME, device_name),
            ])
        ).issuer_name(
            ISSUER
        ).public_key(
            key.public_key()
        ).serial_number(
            x509.random_serial_number()
        ).not_valid_before(
            datetime.datetime.utcnow()
        ).not_valid_after(
            VALID_UNTIL
        ).add_extension(
            x509.BasicConstraints(ca=False, path_length=None)
            ,critical=False
        ).add_extension(
            x509.KeyUsage(
                digital_signature=True
                ,key_encipherment=True
                ,content_commitment=True
                ,data_encipherment=False
                ,key_agreement=False
                ,key_cert_sign=False
                ,crl_sign=False
                ,encipher_only=False
                ,decipher_only=False
                )
            ,critical=True
        ).sign(ca_key, hashes.SHA256(), default_backend())

    return {'cert': cert, 'key': key}


"""
Create a cert and private key for a new device that will connect to AWS IoT.
"""

from os import path
from uuid import uuid4
import json

from cryptography import x509
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.serialization import load_pem_private_key

from ..file_system import configpath
from ..crypto import serialize_key_cert_pair, generate_csr, generate_signed_cert


def provision(country='US', days=365, key_size=2048, **kwargs):
    """
    Creates a new device key and certificate that is signed by a custom root CA.
    """
    thing_type = kwargs.get('thing_type')
    root_ca_path = kwargs.get('root_ca_path')
    root_ca_key_path = kwargs.get('root_ca_key_path')
    common_name = kwargs.get('common_name', str(uuid4()))
    organization = kwargs.get('organization')
    state = kwargs.get('state')
    locality = kwargs.get('locality')
    unit = kwargs.get('unit')

    if thing_type != None:
        ca_path = path.join(configpath, thing_type, 'rootCA.crt')
        ca_key_path = path.join(configpath, thing_type, 'rootCA.key')
    else:
        if (root_ca_path == None) or (root_ca_key_path == None):
            raise Exception('If thing-type is not provided, --root-ca-path and --root-ca-key-path must be set.')
        ca_path = root_ca_path
        ca_key_path = root_ca_key_path


    with open(ca_path, 'rb') as f:
        serialized_ca_cert = f.read()

    ca_cert = x509.load_pem_x509_certificate(serialized_ca_cert, default_backend())

    with open(ca_key_path, 'rb') as f:
        ca_key = load_pem_private_key(f.read(), None, default_backend())

    csr_pair = generate_csr(key_size, common_name, None, organization, country, state, locality, unit)

    cert = generate_signed_cert(csr_pair['csr'], {'cert': ca_cert, 'key': ca_key}, days)['cert']

    serialized_pair = serialize_key_cert_pair({'cert': cert, 'key': csr_pair['key']})

    output = {
        'deviceCert': serialized_pair['cert'],
        'deviceCertPlusCACert': serialized_pair['cert'] + serialized_ca_cert.decode('utf-8'),
        'deviceKey': serialized_pair['key'],
        'commonName': common_name
    }

    return output


import os
import csv

basepath = os.path.dirname(os.path.abspath(__file__))

def get_home_directory():
    home = os.curdir

    if 'HOME' in os.environ:
        home = os.environ['HOME']
    elif os.name == 'posix':
        home = os.path.expanduser("~/")
    elif os.name == 'nt':
        if 'HOMEPATH' in os.environ and 'HOMEDRIVE' in os.environ:
            home = os.environ['HOMEDRIVE'] + os.environ['HOMEPATH']
    else:
        home = os.environ['HOMEPATH']

    return home

configpath = os.path.join(get_home_directory(), '.wintergreen')

def mkdir(base=".", subdir="tmp"):
    """
    Side Effect!
        Creates a directory in at base/subdir
    """
    directory = os.path.join(base, subdir)
    if not os.path.exists(directory):
        os.mkdir(directory)
    return directory

def write_output(output_dir, common_name, output):
    """
    Side Effects!
        Writes deviceKey-<common_name>.key
        Writes deviceCert-<common_name>.crt
        Writes deviceCertPlusCACert-<common_name>.crt
    """
    device_dir = mkdir(base=output_dir, subdir=common_name)
    write_device_key(device_dir, common_name, output)
    write_device_cert(device_dir, common_name, output)
    write_device_combined_cert(device_dir, common_name, output)

def record_output(date, output_dir, generated):
    """
    CommonName = UUID
    DateProvisioned = today, ISO-8601
    CommonName, DateProvisioned
    """
    record_path = os.path.join(output_dir, 'output.csv')
    with open(record_path, 'w') as csvfile:
        fieldnames = ['DateProvisioned', 'CommonName']
        writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
        writer.writeheader()
        for g in generated:
            writer.writerow(
                {
                    'DateProvisioned': date,
                    'CommonName': g,
                }
            )


def write_device_key(device_dir, common_name, output):
    write_file(device_dir, common_name, output, "deviceKey", "key")

def write_device_cert(device_dir, common_name, output):
    write_file(device_dir, common_name, output, "deviceCert", "crt")

def write_device_combined_cert(device_dir, common_name, output):
    write_file(device_dir, common_name, output, "deviceCertPlusCACert", "crt")

def write_file(device_dir, common_name, output, file_name, file_type):
    path = os.path.join(device_dir, '{}-{}.{}'.format(file_name, common_name, file_type))
    with open(path,'w') as myFile:
        myFile.writelines([line for line in output[file_name]])



import boto3
import json

IAM = boto3.client('iam')

TRUST_POLICY = {
    "Version": "2012-10-17",
    "Statement": [
        {
            "Sid": "",
            "Effect": "Allow",
            "Principal": {
            "Service": "iot.amazonaws.com"
            },
            "Action": "sts:AssumeRole"
        }
    ]
}

def create_jitp_role(project_name):
    try:
        response = IAM.create_role(
            RoleName='{project_name}-JITPRole'.format(project_name=project_name),
            AssumeRolePolicyDocument=json.dumps(TRUST_POLICY),
            Description='Just-in-time-provisioning Role for AWS IoT for {project_name}'.format(project_name=project_name),
        )
        role_name = response.get("Role", {}).get("RoleName", None)
        role_arn = response.get("Role", {}).get("Arn", None)

        managed_policies = [
            "arn:aws:iam::aws:policy/service-role/AWSIoTThingsRegistration",
            "arn:aws:iam::aws:policy/service-role/AWSIoTLogging",
            "arn:aws:iam::aws:policy/service-role/AWSIoTRuleActions"
        ]

        for policy in managed_policies:
            IAM.attach_role_policy(RoleName=role_name, PolicyArn=policy)
        return {"role_name": role_name, "role_arn": role_arn}

    except IAM.exceptions.EntityAlreadyExistsException as e:
        response = IAM.get_role(RoleName='{project_name}-JITPRole'.format(project_name=project_name))
        role_name = response.get("Role", {}).get("RoleName", None)
        role_arn = response.get("Role", {}).get("Arn", None)
        return {"role_name": role_name, "role_arn": role_arn}

    except Exception as e:
        raise


"""
Entrypoint for the Wintergreen CLI
"""
import datetime
import json
import os
from uuid import uuid4, UUID

import click

from .initialize import init as init_impl
from .provision import provision as provision_impl
from .file_system import write_output, mkdir, record_output, configpath

basepath = os.path.dirname(os.path.abspath(__file__))

@click.group()
def cli():
    pass

@click.command()
@click.option('-t', '--thing-type', required=True, help='AWS IoT thing type')
@click.option('-c', '--country', required=True, type=str, help='Country code (2-letter) to use for provisioning certs i.e. US', default='US')
# @click.option('--template', help='Optional path to custom just-in-time provisioning template https://docs.aws.amazon.com/iot/latest/developerguide/jit-provisioning.html')
# @click.option('-p', '--policy', help='Optional path to custom aws iot policy https://docs.aws.amazon.com/iot/latest/developerguide/iot-policies.html')
@click.option('-f', '--fqdn', help='Domain of the organization to use in the CA e.g. verypossible.com')
@click.option('-o', '--organization', help='Organization name i.e. Very')
@click.option('-s', '--state', type=str, help='State or Province code (2-letter) to use for provisioning certs i.e. TN')
@click.option('-l', '--locality', type=str, help='Locality to use for provisioning certs i.e. Chattanooga')
@click.option('-u', '--unit', type=str, help='Organizational unit to use for provisioning certs i.e. Engineering')
@click.option('-d', '--days', type=int, show_default=True, default=365, help='Days until CA will expire')
@click.option('-k', '--key-size', type=int, show_default=True, default=2048)
@click.option('--log-level', type=str, default='DEBUG')
def init(thing_type, country, fqdn, organization, state, locality, unit, days, key_size, log_level):
    output = init_impl(
        thing_type,
        country,
        fqdn=fqdn,
        organization=organization,
        state=state,
        locality=locality,
        unit=unit,
        days=days,
        key_size=key_size,
        log_level=log_level
    )

    click.echo(json.dumps(output, indent=4))

@click.command()
@click.option('-t', '--thing-type', help='Same as the thing-type that was chosen in the `init` step.')
@click.option('--root-ca-path', type=click.Path(), help='Required if thing-type from a previous run of `init` isn\'nt provided.')
@click.option('--root-ca-key-path', type=click.Path(), help='Required if thing-type from a previous run of `init` isn\'nt provided.')
@click.option('-c', '--common-name', default=str(uuid4()), help='Defaults to a UUIDv4')
@click.option('-c', '--country', required=True, default='US')
@click.option('-o', '--organization')
@click.option('-s', '--state')
@click.option('-l', '--locality')
@click.option('-u', '--unit')
@click.option('-d', '--days', help='Days until certificate will expire', type=int, default=365, show_default=True)
@click.option('-k', '--key-size', type=int, show_default=True, default=2048)
@click.option('-n', '--number', type=int, show_default=True, default=1, help='Number of certificates to provision')
@click.option('-p', '--print-output', type=bool, default=False, help='Print results to console?')
def provision(thing_type, root_ca_path, root_ca_key_path, common_name, country, organization, state, locality, unit, days, key_size, number, print_output):
    date = datetime.datetime.utcnow().isoformat()
    dated_dir = os.path.join(thing_type, date)
    output_dir = mkdir(base=configpath, subdir=dated_dir)
    if number > 1 and not is_uuid(common_name):
        click.echo("Skipping redundant cert creations, forcing n=1")
        number = 1

    generated = []
    for n in range(number):
        # We need to change the common_name when generating more than one cert
        if n > 0:
            common_name = str(uuid4())
        output = provision_impl(
            country,
            thing_type=thing_type,
            root_ca_path=root_ca_path,
            root_ca_key_path=root_ca_key_path,
            common_name=common_name,
            organization=organization,
            state=state,
            locality=locality,
            unit=unit,
            days=days,
            key_size=key_size
        )
        generated.append(common_name)
        write_output(output_dir, common_name, output)
        if print_output:
            click.echo(json.dumps(output, indent=4))
    click.echo("Generated {0} certificate(s)".format(len(generated)))
    record_output(date, output_dir, generated)

def is_uuid(common_name):
    try:
        UUID(common_name)
        return True
    except ValueError:
        return False

cli.add_command(init)
cli.add_command(provision)

if __name__ == "__main__":
    cli()



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['wintergreen', 'wintergreen.initialize', 'wintergreen.provision']

package_data = \
{'': ['*'], 'wintergreen': ['templates/*']}

install_requires = \
['boto3>=1.9,<2.0',
 'click>=7.0,<8.0',
 'cryptography>=2.6,<3.0',
 'mock>=3.0,<4.0',
 'pytest>=4.5,<5.0']

entry_points = \
{'console_scripts': ['wintergreen = wintergreen.cli:cli']}

setup_kwargs = {
    'name': 'wintergreen',
    'version': '0.1.0',
    'description': 'A tool for dealing with AWS IoT certificates',
    'long_description': '# THIS REPOSITORY IS A WORK IN PROGRESS\n**IF YOU SEE THIS MESSAGE DO NOT UNDER ANY CIRCUMSTANCE DELETE INFRASTRUCTURE HOPING THAT WINTERGREEN CAN REBUILD IT**\nWe\'re working on it.\n\n## Purpose\n\nAutomate device registry in AWS IoT because it sucks. `wintergreen` is a command line utility that provides mechanisms to automate cumbersome tasks related to signing and registering AWS IoT Thing device certificates.\n\n## Install\nThis project uses [poetry](https://poetry.eustace.io/) to manage and install dependencies.\n```\npoetry install\n```\n\nThis will install the required dependencies as well as the `wintergreen` cli, which can be invoked by running `poetry run wintergreen`\n\n\n### Using with direnv\nFollow this [layout_poetry](https://github.com/direnv/direnv/wiki/Python#poetry) example\n\n## Run\n\n## Initialize\nInitialization means\n- Creating a ThingType\n- Creating a RootCA cert/key pair\n- Registering that RootCA with AWS\n- Creating a Just-in-time Provisioning role\n\nExample:\n`poetry run wintergreen init -t my-cool-thing`\nCheck your `~/.wintergreen` directory to see a subdirectory for `my-cool-thing`\n\n```bash\n$  wintergreen init --help\nUsage: wintergreen init [OPTIONS]\n\n  Creates a custom root CA and registers it with the cloud provider for\n  automatic provisioning.\n\nOptions:\n  -t, --thing-type TEXT       AWS IoT thing type  [required]\n  -c, --country TEXT          Country code (2-letter) to use for provisioning\n                              certs i.e. US  [required]\n  -f, --fqdn TEXT             Domain of the organization to use in the CA e.g.\n                              verypossible.com\n  -o, --organization TEXT     Organization name i.e. Very\n  -s, --state TEXT            State or Province code (2-letter) to use for\n                              provisioning certs i.e. TN\n  -l, --locality TEXT         Locality to use for provisioning certs i.e.\n                              Chattanooga\n  -u, --unit TEXT             Organizational unit to use for provisioning\n                              certs i.e. Engineering\n  -d, --root-ca-days INTEGER  Days until CA will expire  [default: 365]\n  -k, --key-size INTEGER      [default: 2048]\n  --log-level TEXT\n  --help                      Show this message and exit.\n\nOutputs:\n{\n    jitpRoleArn: "arn:to:role",\n    rootCACertPath: "${HOME}/.wintergreen/things/Widget/rootCA.crt",\n    rootCAKeyPath: "${HOME}/.wintergreen/things/Widget/rootCA.key",\n    certificateArn:"arn:aws:iot:account:region:cacert/1234"\n}\n```\n\n## Provision a new device\n\nProvisioning a device means generating a certificate/key pair to be used for the device to authenticate with AWS IoT.\n\n```bash\n$ wintergreen provision --help\nUsage: wintergreen provision [OPTIONS]\n\n  Creates a new device key and certificate that is signed by a custom root\n  CA.\n\nOptions:\n  -t, --thing-type TEXT    Same as the thing-type that was chosen in the\n                           `init` step.\n  --root-ca-path PATH      Required if thing-type from a previous run of\n                           `init` is not provided.\n  --root-ca-key-path PATH  Required if thing-type from a previous run of\n                           `init` is not provided.\n  -c, --common-name TEXT   Defaults to a UUIDv4\n  -c, --country TEXT       [required]\n  -o, --organization TEXT\n  -s, --state TEXT\n  -l, --locality TEXT\n  -u, --unit TEXT\n  -d, --days INTEGER       Days until certificate will expire  [default: 365]\n  -k, --key-size INTEGER   [default: 2048]\n  -n, --number             Number of certificates to provision\n  --help                   Show this message and exit.\n\nOutputs:\n{\n    deviceCert: "-----BEGIN CERTIFICATE-----....",\n    deviceCertPlusCACert: "-----BEGIN CERTIFICATE-----....",\n    deviceKey: "------BEGIN RSA PRIVATE KEY----...",\n    commonName: "thing-name"\n}\n```\n\nThese will be written as files scoped by date into your `.wintergreen` directory. If you use the `-n` argument, it will generate a unique uuid for each cert and put all of the certs generated in their own folder. You\'ll also see an output.csv which contains a table of all the certs generated on this run.\n\n### Example provisioning cert example (AWS)\nAfter running the `provision` command the cert/key will be saved in a directory named after the thing-type as `deviceCertPlusCACert-{common_name}.crt` and `deviceKey-{common_name}.key`\nYou\'ll also need to grab the `aws-root-ca.pem` from [AWS Trust](https://www.amazontrust.com/repository/).\nLastly to test, you\'ll need to get your ATS endpoint by running `aws iot describe-endpoint --endpoint-type iot:Data-ATS --region us-east-1`\nIf you have an mqtt client (`brew install mosquitto`) installed, you can verify that a generated device will provision correctly by running something like\n\n```\nmosquitto_pub --cafile aws-root-ca.pem \\\n --cert path/to/deviceCertPlusCACert-my-thing-common-name.crt \\\n --key path/to/deviceKey-my-thing-common-name.key \\\n -h your-endpoint-for-ats.iot.us-east-1.amazonaws.com \\\n --tls-version tlsv1.2 \\\n -p 8883 \\\n -q 1 \\\n -t /my-thing-type/my-thing-common-name/events \\\n -I anyclientID -m "{Hello}" -d\n```\n\nEventually, you will see a device `my-thing-common-name` in the Things menu.',
    'author': 'Dan Lindeman',
    'author_email': 'lindemda@gmail.com',
    'url': 'https://github.com/verypossible/wintergreen',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
